/*

 Command-Line Option Parsing

 by Nuno D. Mendes
 $Id: clo.c,v 1.2 2008/10/02 21:38:09 rnl14480 Exp $

 */

#include <stdio.h>
#include <string.h>
#include "clo.h"
#include "debug.h"

void usage(clo_t clo[]) {
	int i = 0;

	printf("\nextractor OPTIONS <fasta-file>\n\n");

	do {
		if (clo[i].has_param)
			printf("-%c PARAM --%s=PARAM\t%s\n", clo[i].short_option,
					clo[i].long_option, clo[i].description);
		else
			printf("-%c --%s\t%s\n", clo[i].short_option, clo[i].long_option,
					clo[i].description);

	} while (!clo[i++].last_option);

	return;

}

boolean_t clo_is_set(clo_t clo[], char c) {
	int i = 0;

	do {
		if (clo[i].short_option == c && clo[i].is_set == 1)
			return 1;
	} while (!clo[i++].last_option);

	return 0;
}

char *clo_param(clo_t * clo, char c) {
	int i = 0;

	do {
		if (clo[i].short_option == c && clo[i].is_set == 1 && clo[i].has_param)
			return clo[i].param;

	} while (!clo[i++].last_option);

	return NULL;

}

int parse_clo(clo_t * clo, int argc, char **argv, int *left, char **option) {
	int i = 1, j = 0, len = 0;
	char *p = NULL;
	boolean_t found = 0;

	if (argc < 2)
		return CLO_TOOFEW;

	*left = argc - 1;
	*option = "";

	message("Parsing command-line argc = %d", argc);

	while (i < argc) {
		j = 0;
		*option = argv[i];
		found = 0;
		if (argv[i][0] == '-') {
			if (argv[i][1] == '-') {
				do {
					len = strlen(clo[j].long_option);
					message("Comparing %d chars of %s and %s", len,
							clo[j].long_option, argv[i] + 2);
					if (!strncmp(clo[j].long_option, argv[i] + 2, len)) {
						found = 1;
						if (clo[j].has_param) {
							p = argv[i];
							p += len;
							if (p[0] != '=')
								return CLO_BADFORMAT;
							p++;
							clo[j].is_set = 1;
							clo[j].param = p;
							i++;
							(*left)--;
							break;
						} else {
							clo[j].is_set = 1;
							i++;
							(*left)--;
							break;
						}
					}
				} while (!clo[j++].last_option);
				if (!found)
					return CLO_UNKNOWN_OPTION;
			} else {
				do {
					if (clo[j].short_option == argv[i][1]) {
						found = 1;
						if (clo[j].has_param)
							if (argv[i + 1][0] != '-') {
								clo[j].is_set = 1;
								clo[j].param = argv[i + 1];
								i += 2;
								*left -= 2;
								break;
							} else
								return CLO_MISSINGPARAM;
						else {
							clo[j].is_set = 1;
							i++;
							(*left)--;
							break;
						}
					}
				} while (!clo[j++].last_option);
				if (!found)
					return CLO_UNKNOWN_OPTION;
			}
		} else
			i++;
	}
	return 0;
}
