#!/usr/bin/perl -w
# $Id: generate_randomized_genomes.pl,v 1.4 2010/07/07 16:20:26 rnl14480 Exp $

use strict;
use lib qw(../lib/);

use BAOBAB::Fasta;
use BAOBAB::HMM;

die "Usage: $0 data-dir\n" unless scalar(@ARGV) >= 1;

my $data = shift;

my @organisms = qw(ada aga planted_metazoa dmel);

my $gsize = 5_000_000;

foreach my $org (@organisms) {

	print "\nProcessing organism $org...\n";
	my $fasta   = get_fasta($org);
	my $handler = get_handler($org);

	print STDERR "Processing Fasta file : $fasta\n";
	my $fobj = BAOBAB::Fasta->new(
		filename       => $fasta,
		header_handler => $handler
	);

	my @sequences = $fobj->sequences;
	@sequences = map { $_->sequence } @sequences;

	my $hmm = BAOBAB::HMM->new();
	print STDERR "Training HMM with genome...";
	$hmm->train( seqs => \@sequences );
	print STDERR "done.\n";

	open( OUT, ">random_$org.nfa" ) or die "Could not open random_$org : $!\n";
	print STDERR "Generating genome...";
	print OUT ">Randomized genome for $org\n";
	print OUT $hmm->generate( len => $gsize ) . "\n\n";
	print STDERR "done.\n";
	close(OUT);
}

sub get_fasta {
	my $o    = shift;
	my %data = (
		ada             => $data . q(darlingi.nfa),
		aga             => $data . q(aga.nfa),
		planted_metazoa => $data . q(planted_metazoa.nfa),
		dmel            => $data . q{dmel.nfa}
	);

	return $data{$o};
}

sub get_handler {

	my $o    = shift;
	my %data = (
		ada => sub { $_ = shift; s/\s//g; return $_; },
		aga             => sub { ( split /:/, shift )[1] },
		planted_metazoa => sub { return 'U' },
		dmel            => sub {
			$_ = shift;
			if (/^arm_(\S+)/) {
				return $1;
			}
			elsif (/^Arm(\S+)/) {
				return $1;
			}
			elsif (/^(\S+)/) {
				return $1;
			}
			else {
				return '??';
			}
		}
	);

	return $data{$o};
}

1;
