/*

 FastA Processing Functions

 by Nuno D. Mendes
 $Id: fasta.c,v 1.8 2009/06/12 19:35:03 rnl14480 Exp $

 */

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>

#include "sequences.h"
#include "util.h"
#include "debug.h"

#define BUFFERLEN 	1024*1024	/* original was 1000000 */
#define HBUFFERLEN	50
#define SBUFFERLEN	1024
#define SEQESTIMATE	100

dataset_t *
parse_fasta(FILE * fasta) {
	char buffer[BUFFERLEN];
	char c = '\0';
	uint seqn = 0, seqestimate = 0;
	char **sequences = NULL, **seqbuffer = NULL, *curseqbuffer = NULL;
	char **headers = NULL, **headbuffer = NULL, *curheadbuffer = NULL;

	int i = 0, j = 0;
	uint state = 0;
	uint seenheader = 0;
	uint totalsize = 0, sextension = 0;

	ssize_t read = 0, seqlen = 0, seqalloc = 0;
	ssize_t headlen = 0, headalloc = 0;

	if (fasta == NULL)
		return NULL;

	/* clear input buffer */
	bzero(buffer, BUFFERLEN);

	/* fill the buffer and process it */
	while ((read = fread(buffer, 1, BUFFERLEN - 1, fasta)) > 0) {
		i = 0;

		/* check character by character until end of buffer */
		while ((c = buffer[i]) != '\0') {
			switch (c) {
			case '>':
				state = 1;
				seenheader = 1;

				/*
				 * number of sequences exceeds estimation,
				 * realloc
				 */
				if (seqn >= seqestimate) {

					seqbuffer = (char **) safe_malloc(sizeof(char *) * (seqn
							+ SEQESTIMATE));

					headbuffer = (char **) safe_malloc(sizeof(char *) * (seqn
							+ SEQESTIMATE));

					for (j = 0; j < seqn; j++) {
						seqbuffer[j] = sequences[j];
						headbuffer[j] = headers[j];
					}

					for (j = seqn; j < seqn + SEQESTIMATE; j++) {
						seqbuffer[j] = NULL;
						headbuffer[j] = NULL;
					}

					if (seqestimate) {
						safe_free(sequences);
						safe_free(headers);
					}
					sequences = seqbuffer;
					headers = headbuffer;
					seqestimate = seqn + SEQESTIMATE;

				}

				/* new sequence, reset seq vars */
				seqn++;
				seqlen = seqalloc = 0;
				headlen = headalloc = 0;
				break;
			case '\n':
				if (state == 1) /* end of header */
					state = 0;
				break;
			case ' ':
				break;
			default:
				if (state == 0) { /* sequence data */
					if (!seenheader)
						error("Invalid FASTA file");

					if (seqlen + 1 >= seqalloc) {

						sextension = !totalsize ? SBUFFERLEN : totalsize / seqn;
						message("...extending sequence allocation by %d bytes",
								sextension);

						curseqbuffer = (char *) safe_malloc(sizeof(char)
								* (seqlen + sextension));

						bzero(curseqbuffer, seqlen + sextension);

						if (seqlen) {
							strcpy(curseqbuffer, sequences[seqn - 1]);
							safe_free(sequences[seqn - 1]);
						}
						sequences[seqn - 1] = curseqbuffer;
						seqalloc = seqlen + sextension;
					}
					sequences[seqn - 1][seqlen++] = buffer[i];
					totalsize++;
				} else { /* header data */
					if (headlen + 1 >= headalloc) {
						curheadbuffer = (char *) safe_malloc(sizeof(char)
								* (headlen + HBUFFERLEN));
						bzero(curheadbuffer, headlen + HBUFFERLEN);

						if (headlen) {
							strcpy(curheadbuffer, headers[seqn - 1]);
							safe_free(headers[seqn - 1]);
						}
						headers[seqn - 1] = curheadbuffer;

						headalloc = headlen + HBUFFERLEN;

					}
					headers[seqn - 1][headlen++] = buffer[i];

				}
			}
			i++;
		}
		bzero(buffer, BUFFERLEN);
	}

	/* wrap up */
	seqbuffer = (char **) safe_malloc(sizeof(char *) * (seqn));
	headbuffer = (char **) safe_malloc(sizeof(char *) * (seqn));

	for (j = 0; j < seqn; j++) {
		seqbuffer[j] = sequences[j];
		headbuffer[j] = headers[j];
	}

	safe_free(sequences);
	safe_free(headers);
	sequences = seqbuffer;
	headers = headbuffer;

	for (i = 0; i < seqn; i++) {

		if (headers[i] != NULL) {
			headlen = strlen(headers[i]) + 1;
			curheadbuffer = (char *) safe_malloc(sizeof(char) * (headlen));
			bzero(curheadbuffer, headlen);

			if (headlen) {
				strcpy(curheadbuffer, headers[i]);
				safe_free(headers[i]);
			}
			headers[i] = curheadbuffer;
		}
		if (sequences[i] != NULL) {
			seqlen = strlen(sequences[i]) + 1;
			curseqbuffer = (char *) safe_malloc(sizeof(char) * (seqlen));
			bzero(curseqbuffer, seqlen);

			if (seqlen) {
				strcpy(curseqbuffer, sequences[i]);
				safe_free(sequences[i]);
			}
			sequences[i] = curseqbuffer;

		}
	}

	return creat_dataset(seqn, sequences, headers);
}
